unit FileArchive;

{       TFileArchive Component 1.0
        Copyright (c) 2003 Wetgos (wetgos@gmx.net)
                                                        }

interface

uses
  Windows, Messages, SysUtils, Classes;

type
  EArchiveException = class(Exception);
  TArchiveEventType = (arError,arDone,arLoading,arSaving,arAdding,arDeleting,arSearching,arExtracting);
  TArchiveEvent = procedure (Sender: TObject; EventType: TArchiveEventType) of object;

  TArchiveFile = class(TCollectionItem)
  public
    Stream: TMemoryStream;
    Name: string;
    procedure LoadFromFile(FileName: string);
    procedure SaveToFile(FileName: string);
    constructor Create(ACollection: TCollection); override;
  end;

  TFileArchive = class(TComponent)
  private
    FOnEvent: TArchiveEvent;
    FStatus: TArchiveEventType;
    function FillStr(const S: string; StrLength: Integer): string;
    function GetSize: Integer;
    procedure ClearEverything;
    procedure SetStatus(const Value: TArchiveEventType);
    procedure SetOnEvent(const Value: TArchiveEvent);
  public
    Files: TCollection;
    procedure LoadFromFile(FileName: string);
    procedure SaveToFile(FileName: string);
    procedure AddToArchive(FileName: string);
    procedure DeleteFromArchive(ArFile: TArchiveFile);
    procedure ExtractFromArchive(ArFile: TArchiveFile; Folder: string);
    procedure ExtractAll(Folder: string);
    function FindFile(FileName: string): TArchiveFile;
    function FileList: TStringList;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Status: TArchiveEventType read FStatus;
    property OnEvent: TArchiveEvent read FOnEvent write SetOnEvent;
    property Size: Integer read GetSize;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Standard', [TFileArchive]);
end;

{ TFileArchive }

procedure TFileArchive.AddToArchive(FileName: string);
var
  ArFile: TArchiveFile;
begin
  if FindFile(ExtractFileName(FileName)) = nil then begin
    SetStatus(arAdding);
    ArFile := Files.Add as TArchiveFile;
    ArFile.LoadFromFile(FileName);
    SetStatus(arDone);
  end else begin
    SetStatus(arError);
    raise EArchiveException.CreateFmt('Archive already contains a file named %s',[ExtractFileName(FileName)]);
  end;
end;

procedure TFileArchive.ClearEverything;
var
  I: Integer;
begin
  for I:=0 to Files.Count-1 do begin
    Files.Items[0].Destroy;
  end;
  FStatus := arDone;
end;

constructor TFileArchive.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Files := TCollection.Create(TArchiveFile);
  ClearEverything;
end;

procedure TFileArchive.DeleteFromArchive(ArFile: TArchiveFile);
begin
  if ArFile = nil then begin
    SetStatus(arError);
    raise EArchiveException.Create('File doesn''t exist');
    Exit;
  end;
  SetStatus(arDeleting);
  ArFile.Destroy;
  SetStatus(arDone);
end;

destructor TFileArchive.Destroy;
begin
  ClearEverything;
  inherited;
end;

procedure TFileArchive.ExtractAll(Folder: string);
var
  I: Integer;
begin
  if not DirectoryExists(Folder) then begin
    SetStatus(arError);
    raise EArchiveException.CreateFmt('Destination folder %s doesn''t exist',[Folder]);
    Exit;
  end;
  for I:=0 to Files.Count-1 do begin
    ExtractFromArchive(TArchiveFile(Files.Items[I]),Folder);
  end;
end;

procedure TFileArchive.ExtractFromArchive(ArFile: TArchiveFile; Folder: string);
var
  OutStream: TFileStream;
  OutName: string;
begin
  if ArFile = nil then begin
    SetStatus(arError);
    raise EArchiveException.Create('File doesn''t exist');
    Exit;
  end;
  if not DirectoryExists(Folder) then begin
    SetStatus(arError);
    raise EArchiveException.CreateFmt('Destination folder %s doesn''t exist',[Folder]);
    Exit;
  end;
  SetStatus(arExtracting);
  OutName := Folder + '\' + ArFile.Name;
  OutStream := TFileStream.Create(OutName,fmCreate or fmOpenWrite);
  try
    OutStream.Seek(0,soFromBeginning);
    ArFile.Stream.Seek(0,soFromBeginning);
    OutStream.CopyFrom(ArFile.Stream,ArFile.Stream.Size);
    SetStatus(arDone);
  finally
    OutStream.Destroy;
  end;
end;

function TFileArchive.FileList: TStringList;
var
  I: Integer;
  ArFile: TArchiveFile;
begin
  Result := TStringList.Create;
  for I:=0 to Files.Count-1 do begin
    ArFile := TArchiveFile(Files.Items[I]);
    Result.Add(Format('%s (%d bytes)',[ArFile.Name,ArFile.Stream.Size]));
  end;
end;

function TFileArchive.FillStr(const S: string; StrLength: Integer): string;
begin
  Result := S;
  while Length(Result) < StrLength do begin
    Result := Result + ' ';
  end;
end;

function TFileArchive.FindFile(FileName: string): TArchiveFile;
var
  I: Integer;
begin
  SetStatus(arSearching);
  Result := nil;
  for I:=0 to Files.Count-1 do begin
    if TArchiveFile(Files.Items[I]).Name = FileName then begin
      Result := TArchiveFile(Files.Items[I]);
      Break;
    end;
  end;
  SetStatus(arDone);
end;

function TFileArchive.GetSize: Integer;
var
  I: Integer;
begin
  Result := 20;
  for I:=0 to Files.Count-1 do begin
    Result := Result + TArchiveFile(Files.Items[I]).Stream.Size + 170;
  end;
end;

procedure TFileArchive.LoadFromFile(FileName: string);
var
  LoadStream: TFileStream;
  ArFile: TArchiveFile;
  I,FileCount,EntrySize,TotalEntries: Integer;
  BufferStr: array[0..255] of Char;
  Header,EntryName: string;
begin
  SetStatus(arLoading);
  if not FileExists(FileName) then begin
    SetStatus(arError);
    raise EArchiveException.CreateFmt('Archive %s doesn''t exist',[FileName]);
    Exit;
  end;
  ClearEverything;

  LoadStream := TFileStream.Create(FileName,fmOpenRead);
  try
    LoadStream.Seek(0,soFromBeginning);

    LoadStream.Read(BufferStr,20);
    SetString(Header,BufferStr,20);
    Header := Trim(Header);

    if Header = '' then begin
      raise EArchiveException.CreateFmt('File %s isn''t a valid archive',[FileName]);
      Exit;
    end;

    FileCount := StrToInt(Header);
    if FileCount = 0 then begin
      raise EArchiveException.CreateFmt('Archive %s doesn''t contain any files',[FileName]);
      Exit;
    end;

    TotalEntries := (FileCount * 170) + 20;

    for I:=0 to FileCount-1 do begin
      LoadStream.Seek((I*170) + 20,soFromBeginning);
      LoadStream.Read(BufferStr,150);
      SetString(Header,BufferStr,150);
      Header := Trim(Header);
      EntryName := Header;

      LoadStream.Read(BufferStr,20);
      SetString(Header,BufferStr,20);
      Header := Trim(Header);
      EntrySize := StrToInt(Header);

      ArFile := Files.Add as TArchiveFile;
      ArFile.Stream.Seek(0,soFromBeginning);
      LoadStream.Seek(TotalEntries,soFromBeginning);
      ArFile.Stream.CopyFrom(LoadStream,EntrySize);
      ArFile.Name := EntryName;
      
      TotalEntries := TotalEntries + EntrySize;
    end;

    SetStatus(arDone);
  finally
    LoadStream.Destroy;
  end;
end;

procedure TFileArchive.SaveToFile(FileName: string);
var
  SaveStream: TFileStream;
  Text: string;
  I: Integer;
  ArFile: TArchiveFile;
begin
  if Files.Count = 0 then begin
    SetStatus(arError);
    raise EArchiveException.Create('Archive has contain at least 1 file');
    Exit;
  end;
  SetStatus(arSaving);
  SaveStream := TFileStream.Create(FileName,fmCreate or fmOpenWrite);
  try
    SaveStream.Seek(0,soFromBeginning);

    Text := FillStr(IntToStr(Files.Count),20);
    SaveStream.Write(PChar(Text)^,20);

    for I:=0 to Files.Count-1 do begin
      Text := FillStr(TArchiveFile(Files.Items[I]).Name,150);
      SaveStream.Write(PChar(Text)^,Length(Text));
      Text := FillStr(IntToStr(TArchiveFile(Files.Items[I]).Stream.Size),20);
      SaveStream.Write(PChar(Text)^,Length(Text));
    end;


    for I:=0 to Files.Count-1 do begin
      ArFile := TArchiveFile(Files.Items[I]);
      ArFile.Stream.Seek(0,soFromBeginning);
      SaveStream.CopyFrom(ArFile.Stream,ArFile.Stream.Size);
    end;

    SetStatus(arDone);
  finally
    SaveStream.Destroy;
  end;
end;

procedure TFileArchive.SetOnEvent(const Value: TArchiveEvent);
begin
  FOnEvent := Value;
end;

procedure TFileArchive.SetStatus(const Value: TArchiveEventType);
begin
  FStatus := Value;
  if Assigned(FOnEvent) then
    FOnEvent(Self,FStatus);
end;

{ TArchiveFile }

constructor TArchiveFile.Create(ACollection: TCollection);
begin
  inherited Create(ACollection);
  Stream := TMemoryStream.Create;
end;

procedure TArchiveFile.LoadFromFile(FileName: string);
begin
  if not FileExists(FileName) then begin
    raise EArchiveException.CreateFmt('File %s doesn''t exist',[FileName]);
    Exit;
  end;
  Stream.LoadFromFile(FileName);
  Name := ExtractFileName(FileName);
end;

procedure TArchiveFile.SaveToFile(FileName: string);
begin
  Stream.SaveToFile(FileName);
end;

end.
 